/****************************************************************************
 *
 *  Copyright (C) 2006 by yetist <yetist@gmail.com>.
 *
 *  文 件 名:gtkchinesecalendar.h
 *  版    本:0.1
 *  创建时间:2006年12月31日 星期日 13时52分28秒
 *  描    述:Chinese Calendar Widget for gtk
 *
 * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
 *****************************************************************************/

#ifndef __GTK_CHINESE_CALENDAR_H__
#define __GTK_CHINESE_CALENDAR_H__

#include <gtk/gtk.h>


G_BEGIN_DECLS

#define GTK_TYPE_CHINESE_CALENDAR                  (gtk_chinese_calendar_get_type ())
#define GTK_CHINESE_CALENDAR(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_CHINESE_CALENDAR, GtkChineseCalendar))
#define GTK_CHINESE_CALENDAR_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_CHINESE_CALENDAR, GtkChineseCalendarClass))
#define GTK_IS_CHINESE_CALENDAR(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_CHINESE_CALENDAR))
#define GTK_IS_CHINESE_CALENDAR_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_CHINESE_CALENDAR))
#define GTK_CHINESE_CALENDAR_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_CHINESE_CALENDAR, GtkChineseCalendarClass))


typedef struct _GtkChineseCalendar	      GtkChineseCalendar;
typedef struct _GtkChineseCalendarClass       GtkChineseCalendarClass;

typedef struct _GtkChineseCalendarPrivate     GtkChineseCalendarPrivate;

typedef enum
{
  GTK_CHINESE_CALENDAR_SHOW_HEADING		= 1 << 0,
  GTK_CHINESE_CALENDAR_SHOW_DAY_NAMES		= 1 << 1,
  GTK_CHINESE_CALENDAR_NO_MONTH_CHANGE		= 1 << 2,
  GTK_CHINESE_CALENDAR_SHOW_WEEK_NUMBERS	= 1 << 3,
  GTK_CHINESE_CALENDAR_WEEK_START_MONDAY	= 1 << 4,
  GTK_CHINESE_CALENDAR_SHOW_LUNAR_DATE          = 1 << 5
} GtkChineseCalendarDisplayOptions;

struct _GtkChineseCalendar
{
  GtkWidget widget;
  
  GtkStyle  *header_style;
  GtkStyle  *label_style;
  
  gint month;
  gint year;
  gint selected_day;
  
  gint day_month[6][7];
  gint day[6][7];
  
  gint num_marked_dates;
  gint marked_date[31];
  GtkChineseCalendarDisplayOptions  display_flags;
  GdkColor marked_date_color[31];
  
  GdkGC *gc;			/* unused */
  GdkGC *xor_gc;		/* unused */

  gint focus_row;
  gint focus_col;

  gint highlight_row;
  gint highlight_col;
  
  GtkChineseCalendarPrivate *priv;
  gchar grow_space [32];

  /* Padding for future expansion */
  void (*_gtk_reserved1) (void);
  void (*_gtk_reserved2) (void);
  void (*_gtk_reserved3) (void);
  void (*_gtk_reserved4) (void);
};

struct _GtkChineseCalendarClass
{
  GtkWidgetClass parent_class;
  
  /* Signal handlers */
  void (* month_changed)		(GtkChineseCalendar *calendar);
  void (* day_selected)			(GtkChineseCalendar *calendar);
  void (* day_selected_double_click)	(GtkChineseCalendar *calendar);
  void (* prev_month)			(GtkChineseCalendar *calendar);
  void (* next_month)			(GtkChineseCalendar *calendar);
  void (* prev_year)			(GtkChineseCalendar *calendar);
  void (* next_year)			(GtkChineseCalendar *calendar);
  
};


GType	   gtk_chinese_calendar_get_type	(void) G_GNUC_CONST;
GtkWidget* gtk_chinese_calendar_new		(void);

gboolean   gtk_chinese_calendar_select_month	(GtkChineseCalendar *calendar, 
					 guint	      month,
					 guint	      year);
void	   gtk_chinese_calendar_select_day	(GtkChineseCalendar *calendar,
					 guint	      day);

gboolean   gtk_chinese_calendar_mark_day	(GtkChineseCalendar *calendar,
					 guint	      day);
gboolean   gtk_chinese_calendar_unmark_day	(GtkChineseCalendar *calendar,
					 guint	      day);
void	   gtk_chinese_calendar_clear_marks	(GtkChineseCalendar *calendar);


void	   gtk_chinese_calendar_set_display_options (GtkChineseCalendar    	      *calendar,
					     GtkChineseCalendarDisplayOptions flags);
GtkChineseCalendarDisplayOptions
           gtk_chinese_calendar_get_display_options (GtkChineseCalendar   	      *calendar);
#ifndef GTK_DISABLE_DEPRECATED
void	   gtk_chinese_calendar_display_options (GtkChineseCalendar		  *calendar,
					 GtkChineseCalendarDisplayOptions flags);
#endif

void	   gtk_chinese_calendar_get_date	(GtkChineseCalendar *calendar, 
					 guint	     *year,
					 guint	     *month,
					 guint	     *day);
#ifndef GTK_DISABLE_DEPRECATED
void	   gtk_chinese_calendar_freeze		(GtkChineseCalendar *calendar);
void	   gtk_chinese_calendar_thaw		(GtkChineseCalendar *calendar);
#endif

G_END_DECLS

#endif /* __GTK_CHINESE_CALENDAR_H__ */
        
